@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 import base64
 import os
 
@@ -6,7 +6,7 @@ if __name__ == "__main__":
     # create font data chunk for embedding
     font = "Tests/images/courB08"
     print("    f._load_pilfont_data(")
-    print("         # %s" % os.path.basename(font))
+    print(f"         # {os.path.basename(font)}")
     print("         BytesIO(base64.decodestring(b'''")
     with open(font + ".pil", "rb") as fp:
         print(base64.b64encode(fp.read()).decode())
