@@ -2,9 +2,11 @@ import ctypes
 import os
 import subprocess
 import sys
-from distutils import ccompiler, sysconfig
+import sysconfig
 
 import pytest
+from setuptools.command.build_ext import new_compiler
+
 from PIL import Image
 
 from .helper import assert_image_equal, hopper, is_win32, on_ci
@@ -15,11 +17,17 @@ if os.environ.get("PYTHONOPTIMIZE") == "2":
     cffi = None
 else:
     try:
-        from PIL import PyAccess
         import cffi
+
+        from PIL import PyAccess
     except ImportError:
         cffi = None
 
+try:
+    import numpy
+except ImportError:
+    numpy = None
+
 
 class AccessTest:
     # initial value
@@ -63,6 +71,10 @@ class TestImagePutPixel(AccessTest):
         pix1 = im1.load()
         pix2 = im2.load()
 
+        for x, y in ((0, "0"), ("0", 0)):
+            with pytest.raises(TypeError):
+                pix1[x, y]
+
         for y in range(im1.size[1]):
             for x in range(im1.size[0]):
                 pix2[x, y] = pix1[x, y]
@@ -106,6 +118,13 @@ class TestImagePutPixel(AccessTest):
 
         assert_image_equal(im1, im2)
 
+    @pytest.mark.skipif(numpy is None, reason="NumPy not installed")
+    def test_numpy(self):
+        im = hopper()
+        pix = im.load()
+
+        assert pix[numpy.int32(1), numpy.int32(2)] == (18, 20, 59)
+
 
 class TestImageGetPixel(AccessTest):
     @staticmethod
@@ -125,14 +144,13 @@ class TestImageGetPixel(AccessTest):
         im.putpixel((0, 0), c)
         assert (
             im.getpixel((0, 0)) == c
-        ), "put/getpixel roundtrip failed for mode {}, color {}".format(mode, c)
+        ), f"put/getpixel roundtrip failed for mode {mode}, color {c}"
 
         # check putpixel negative index
         im.putpixel((-1, -1), c)
-        assert im.getpixel((-1, -1)) == c, (
-            "put/getpixel roundtrip negative index failed for mode %s, color %s"
-            % (mode, c)
-        )
+        assert (
+            im.getpixel((-1, -1)) == c
+        ), f"put/getpixel roundtrip negative index failed for mode {mode}, color {c}"
 
         # Check 0
         im = Image.new(mode, (0, 0), None)
@@ -150,11 +168,11 @@ class TestImageGetPixel(AccessTest):
         im = Image.new(mode, (1, 1), c)
         assert (
             im.getpixel((0, 0)) == c
-        ), "initial color failed for mode {}, color {} ".format(mode, c)
+        ), f"initial color failed for mode {mode}, color {c} "
         # check initial color negative index
         assert (
             im.getpixel((-1, -1)) == c
-        ), "initial color failed with negative index for mode %s, color %s " % (mode, c)
+        ), f"initial color failed with negative index for mode {mode}, color {c} "
 
         # Check 0
         im = Image.new(mode, (0, 0), c)
@@ -325,6 +343,57 @@ class TestCffi(AccessTest):
             assert im.convert("RGB").getpixel((0, 0)) == (255, 0, 0)
 
 
+class TestImagePutPixelError(AccessTest):
+    IMAGE_MODES1 = ["L", "LA", "RGB", "RGBA"]
+    IMAGE_MODES2 = ["I", "I;16", "BGR;15"]
+    INVALID_TYPES = ["foo", 1.0, None]
+
+    @pytest.mark.parametrize("mode", IMAGE_MODES1)
+    def test_putpixel_type_error1(self, mode):
+        im = hopper(mode)
+        for v in self.INVALID_TYPES:
+            with pytest.raises(TypeError, match="color must be int or tuple"):
+                im.putpixel((0, 0), v)
+
+    @pytest.mark.parametrize(
+        ("mode", "band_numbers", "match"),
+        (
+            ("L", (0, 2), "color must be int or single-element tuple"),
+            ("LA", (0, 3), "color must be int, or tuple of one or two elements"),
+            (
+                "RGB",
+                (0, 2, 5),
+                "color must be int, or tuple of one, three or four elements",
+            ),
+        ),
+    )
+    def test_putpixel_invalid_number_of_bands(self, mode, band_numbers, match):
+        im = hopper(mode)
+        for band_number in band_numbers:
+            with pytest.raises(TypeError, match=match):
+                im.putpixel((0, 0), (0,) * band_number)
+
+    @pytest.mark.parametrize("mode", IMAGE_MODES2)
+    def test_putpixel_type_error2(self, mode):
+        im = hopper(mode)
+        for v in self.INVALID_TYPES:
+            with pytest.raises(
+                TypeError, match="color must be int or single-element tuple"
+            ):
+                im.putpixel((0, 0), v)
+
+    @pytest.mark.parametrize("mode", IMAGE_MODES1 + IMAGE_MODES2)
+    def test_putpixel_overflow_error(self, mode):
+        im = hopper(mode)
+        with pytest.raises(OverflowError):
+            im.putpixel((0, 0), 2 ** 80)
+
+    def test_putpixel_unrecognized_mode(self):
+        im = hopper("BGR;15")
+        with pytest.raises(ValueError, match="unrecognized image mode"):
+            im.putpixel((0, 0), 0)
+
+
 class TestEmbeddable:
     @pytest.mark.skipif(
         not is_win32() or on_ci(),
@@ -359,13 +428,12 @@ int main(int argc, char* argv[])
                 % sys.prefix.replace("\\", "\\\\")
             )
 
-        compiler = ccompiler.new_compiler()
-        compiler.add_include_dir(sysconfig.get_python_inc())
+        compiler = new_compiler()
+        compiler.add_include_dir(sysconfig.get_config_var("INCLUDEPY"))
 
-        libdir = sysconfig.get_config_var(
-            "LIBDIR"
-        ) or sysconfig.get_python_inc().replace("include", "libs")
-        print(libdir)
+        libdir = sysconfig.get_config_var("LIBDIR") or sysconfig.get_config_var(
+            "INCLUDEPY"
+        ).replace("include", "libs")
         compiler.add_library_dir(libdir)
         objects = compiler.compile(["embed_pil.c"])
         compiler.link_executable(objects, "embed_pil")
