@@ -2,18 +2,18 @@ import pytest
 
 from PIL import Image
 
-from .helper import assert_image, assert_image_similar, hopper, is_ppc64le
+from .helper import assert_image_similar, hopper, is_ppc64le
 
 
 def test_sanity():
     image = hopper()
     converted = image.quantize()
-    assert_image(converted, "P", converted.size)
+    assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 10)
 
     image = hopper()
     converted = image.quantize(palette=hopper("P"))
-    assert_image(converted, "P", converted.size)
+    assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 60)
 
 
@@ -27,7 +27,7 @@ def test_libimagequant_quantize():
             pytest.skip("libimagequant support not available")
         else:
             raise
-    assert_image(converted, "P", converted.size)
+    assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 15)
     assert len(converted.getcolors()) == 100
 
@@ -35,7 +35,7 @@ def test_libimagequant_quantize():
 def test_octree_quantize():
     image = hopper()
     converted = image.quantize(100, Image.FASTOCTREE)
-    assert_image(converted, "P", converted.size)
+    assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 20)
     assert len(converted.getcolors()) == 100
 
@@ -52,7 +52,7 @@ def test_quantize():
     with Image.open("Tests/images/caption_6_33_22.png") as image:
         image = image.convert("RGB")
     converted = image.quantize()
-    assert_image(converted, "P", converted.size)
+    assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 1)
 
 
@@ -62,7 +62,8 @@ def test_quantize_no_dither():
         palette = palette.convert("P")
 
     converted = image.quantize(dither=0, palette=palette)
-    assert_image(converted, "P", converted.size)
+    assert converted.mode == "P"
+    assert converted.palette.palette == palette.palette.palette
 
 
 def test_quantize_dither_diff():
@@ -76,6 +77,13 @@ def test_quantize_dither_diff():
     assert dither.tobytes() != nodither.tobytes()
 
 
+def test_colors():
+    im = hopper()
+    colors = 2
+    converted = im.quantize(colors)
+    assert len(converted.palette.palette) == colors * len("RGB")
+
+
 def test_transparent_colors_equal():
     im = Image.new("RGBA", (1, 2), (0, 0, 0, 0))
     px = im.load()
@@ -84,3 +92,20 @@ def test_transparent_colors_equal():
     converted = im.quantize()
     converted_px = converted.load()
     assert converted_px[0, 0] == converted_px[0, 1]
+
+
+@pytest.mark.parametrize(
+    "method, color",
+    (
+        (Image.MEDIANCUT, (0, 0, 0)),
+        (Image.MAXCOVERAGE, (0, 0, 0)),
+        (Image.FASTOCTREE, (0, 0, 0)),
+        (Image.FASTOCTREE, (0, 0, 0, 0)),
+    ),
+)
+def test_palette(method, color):
+    im = Image.new("RGBA" if len(color) == 4 else "RGB", (1, 1), color)
+
+    converted = im.quantize(method=method)
+    converted_px = converted.load()
+    assert converted_px[0, 0] == converted.palette.colors[color]
