@@ -3,12 +3,19 @@ Tests for resize functionality.
 """
 from itertools import permutations
 
+import pytest
+
 from PIL import Image
 
-from .helper import PillowTestCase, hopper
+from .helper import (
+    assert_image_equal,
+    assert_image_equal_tofile,
+    assert_image_similar,
+    hopper,
+)
 
 
-class TestImagingCoreResize(PillowTestCase):
+class TestImagingCoreResize:
     def resize(self, im, size, f):
         # Image class independent version of resize.
         im.load()
@@ -29,26 +36,23 @@ class TestImagingCoreResize(PillowTestCase):
         ]:  # exotic mode
             im = hopper(mode)
             r = self.resize(im, (15, 12), Image.NEAREST)
-            self.assertEqual(r.mode, mode)
-            self.assertEqual(r.size, (15, 12))
-            self.assertEqual(r.im.bands, im.im.bands)
+            assert r.mode == mode
+            assert r.size == (15, 12)
+            assert r.im.bands == im.im.bands
 
     def test_convolution_modes(self):
-        self.assertRaises(
-            ValueError, self.resize, hopper("1"), (15, 12), Image.BILINEAR
-        )
-        self.assertRaises(
-            ValueError, self.resize, hopper("P"), (15, 12), Image.BILINEAR
-        )
-        self.assertRaises(
-            ValueError, self.resize, hopper("I;16"), (15, 12), Image.BILINEAR
-        )
+        with pytest.raises(ValueError):
+            self.resize(hopper("1"), (15, 12), Image.BILINEAR)
+        with pytest.raises(ValueError):
+            self.resize(hopper("P"), (15, 12), Image.BILINEAR)
+        with pytest.raises(ValueError):
+            self.resize(hopper("I;16"), (15, 12), Image.BILINEAR)
         for mode in ["L", "I", "F", "RGB", "RGBA", "CMYK", "YCbCr"]:
             im = hopper(mode)
             r = self.resize(im, (15, 12), Image.BILINEAR)
-            self.assertEqual(r.mode, mode)
-            self.assertEqual(r.size, (15, 12))
-            self.assertEqual(r.im.bands, im.im.bands)
+            assert r.mode == mode
+            assert r.size == (15, 12)
+            assert r.im.bands == im.im.bands
 
     def test_reduce_filters(self):
         for f in [
@@ -60,8 +64,8 @@ class TestImagingCoreResize(PillowTestCase):
             Image.LANCZOS,
         ]:
             r = self.resize(hopper("RGB"), (15, 12), f)
-            self.assertEqual(r.mode, "RGB")
-            self.assertEqual(r.size, (15, 12))
+            assert r.mode == "RGB"
+            assert r.size == (15, 12)
 
     def test_enlarge_filters(self):
         for f in [
@@ -73,8 +77,8 @@ class TestImagingCoreResize(PillowTestCase):
             Image.LANCZOS,
         ]:
             r = self.resize(hopper("RGB"), (212, 195), f)
-            self.assertEqual(r.mode, "RGB")
-            self.assertEqual(r.size, (212, 195))
+            assert r.mode == "RGB"
+            assert r.size == (212, 195)
 
     def test_endianness(self):
         # Make an image with one colored pixel, in one channel.
@@ -116,7 +120,7 @@ class TestImagingCoreResize(PillowTestCase):
                     for i, ch in enumerate(resized.split()):
                         # check what resized channel in image is the same
                         # as separately resized channel
-                        self.assert_image_equal(ch, references[channels[i]])
+                        assert_image_equal(ch, references[channels[i]])
 
     def test_enlarge_zero(self):
         for f in [
@@ -128,107 +132,122 @@ class TestImagingCoreResize(PillowTestCase):
             Image.LANCZOS,
         ]:
             r = self.resize(Image.new("RGB", (0, 0), "white"), (212, 195), f)
-            self.assertEqual(r.mode, "RGB")
-            self.assertEqual(r.size, (212, 195))
-            self.assertEqual(r.getdata()[0], (0, 0, 0))
+            assert r.mode == "RGB"
+            assert r.size == (212, 195)
+            assert r.getdata()[0] == (0, 0, 0)
 
     def test_unknown_filter(self):
-        self.assertRaises(ValueError, self.resize, hopper(), (10, 10), 9)
+        with pytest.raises(ValueError):
+            self.resize(hopper(), (10, 10), 9)
+
+    def test_cross_platform(self, tmp_path):
+        # This test is intended for only check for consistent behaviour across
+        # platforms. So if a future Pillow change requires that the test file
+        # be updated, that is okay.
+        im = hopper().resize((64, 64))
+        temp_file = str(tmp_path / "temp.gif")
+        im.save(temp_file)
+
+        with Image.open(temp_file) as reloaded:
+            assert_image_equal_tofile(reloaded, "Tests/images/hopper_resized.gif")
 
 
-class TestReducingGapResize(PillowTestCase):
-    @classmethod
-    def setUpClass(cls):
-        cls.gradients_image = Image.open("Tests/images/radial_gradients.png")
-        cls.gradients_image.load()
+@pytest.fixture
+def gradients_image():
+    with Image.open("Tests/images/radial_gradients.png") as im:
+        im.load()
+    try:
+        yield im
+    finally:
+        im.close()
 
-    def test_reducing_gap_values(self):
-        ref = self.gradients_image.resize((52, 34), Image.BICUBIC, reducing_gap=None)
-        im = self.gradients_image.resize((52, 34), Image.BICUBIC)
-        self.assert_image_equal(ref, im)
 
-        with self.assertRaises(ValueError):
-            self.gradients_image.resize((52, 34), Image.BICUBIC, reducing_gap=0)
+class TestReducingGapResize:
+    def test_reducing_gap_values(self, gradients_image):
+        ref = gradients_image.resize((52, 34), Image.BICUBIC, reducing_gap=None)
+        im = gradients_image.resize((52, 34), Image.BICUBIC)
+        assert_image_equal(ref, im)
 
-        with self.assertRaises(ValueError):
-            self.gradients_image.resize((52, 34), Image.BICUBIC, reducing_gap=0.99)
+        with pytest.raises(ValueError):
+            gradients_image.resize((52, 34), Image.BICUBIC, reducing_gap=0)
 
-    def test_reducing_gap_1(self):
+        with pytest.raises(ValueError):
+            gradients_image.resize((52, 34), Image.BICUBIC, reducing_gap=0.99)
+
+    def test_reducing_gap_1(self, gradients_image):
         for box, epsilon in [
             (None, 4),
             ((1.1, 2.2, 510.8, 510.9), 4),
             ((3, 10, 410, 256), 10),
         ]:
-            ref = self.gradients_image.resize((52, 34), Image.BICUBIC, box=box)
-            im = self.gradients_image.resize(
+            ref = gradients_image.resize((52, 34), Image.BICUBIC, box=box)
+            im = gradients_image.resize(
                 (52, 34), Image.BICUBIC, box=box, reducing_gap=1.0
             )
 
-            with self.assertRaises(AssertionError):
-                self.assert_image_equal(ref, im)
+            with pytest.raises(AssertionError):
+                assert_image_equal(ref, im)
 
-            self.assert_image_similar(ref, im, epsilon)
+            assert_image_similar(ref, im, epsilon)
 
-    def test_reducing_gap_2(self):
+    def test_reducing_gap_2(self, gradients_image):
         for box, epsilon in [
             (None, 1.5),
             ((1.1, 2.2, 510.8, 510.9), 1.5),
             ((3, 10, 410, 256), 1),
         ]:
-            ref = self.gradients_image.resize((52, 34), Image.BICUBIC, box=box)
-            im = self.gradients_image.resize(
+            ref = gradients_image.resize((52, 34), Image.BICUBIC, box=box)
+            im = gradients_image.resize(
                 (52, 34), Image.BICUBIC, box=box, reducing_gap=2.0
             )
 
-            with self.assertRaises(AssertionError):
-                self.assert_image_equal(ref, im)
+            with pytest.raises(AssertionError):
+                assert_image_equal(ref, im)
 
-            self.assert_image_similar(ref, im, epsilon)
+            assert_image_similar(ref, im, epsilon)
 
-    def test_reducing_gap_3(self):
+    def test_reducing_gap_3(self, gradients_image):
         for box, epsilon in [
             (None, 1),
             ((1.1, 2.2, 510.8, 510.9), 1),
             ((3, 10, 410, 256), 0.5),
         ]:
-            ref = self.gradients_image.resize((52, 34), Image.BICUBIC, box=box)
-            im = self.gradients_image.resize(
+            ref = gradients_image.resize((52, 34), Image.BICUBIC, box=box)
+            im = gradients_image.resize(
                 (52, 34), Image.BICUBIC, box=box, reducing_gap=3.0
             )
 
-            with self.assertRaises(AssertionError):
-                self.assert_image_equal(ref, im)
+            with pytest.raises(AssertionError):
+                assert_image_equal(ref, im)
 
-            self.assert_image_similar(ref, im, epsilon)
+            assert_image_similar(ref, im, epsilon)
 
-    def test_reducing_gap_8(self):
+    def test_reducing_gap_8(self, gradients_image):
         for box in [None, (1.1, 2.2, 510.8, 510.9), (3, 10, 410, 256)]:
-            ref = self.gradients_image.resize((52, 34), Image.BICUBIC, box=box)
-            im = self.gradients_image.resize(
+            ref = gradients_image.resize((52, 34), Image.BICUBIC, box=box)
+            im = gradients_image.resize(
                 (52, 34), Image.BICUBIC, box=box, reducing_gap=8.0
             )
 
-            self.assert_image_equal(ref, im)
+            assert_image_equal(ref, im)
 
-    def test_box_filter(self):
+    def test_box_filter(self, gradients_image):
         for box, epsilon in [
             ((0, 0, 512, 512), 5.5),
             ((0.9, 1.7, 128, 128), 9.5),
         ]:
-            ref = self.gradients_image.resize((52, 34), Image.BOX, box=box)
-            im = self.gradients_image.resize(
-                (52, 34), Image.BOX, box=box, reducing_gap=1.0
-            )
+            ref = gradients_image.resize((52, 34), Image.BOX, box=box)
+            im = gradients_image.resize((52, 34), Image.BOX, box=box, reducing_gap=1.0)
 
-            self.assert_image_similar(ref, im, epsilon)
+            assert_image_similar(ref, im, epsilon)
 
 
-class TestImageResize(PillowTestCase):
+class TestImageResize:
     def test_resize(self):
         def resize(mode, size):
             out = hopper(mode).resize(size)
-            self.assertEqual(out.mode, mode)
-            self.assertEqual(out.size, size)
+            assert out.mode == mode
+            assert out.size == size
 
         for mode in "1", "P", "L", "RGB", "I", "F":
             resize(mode, (112, 103))
@@ -236,13 +255,18 @@ class TestImageResize(PillowTestCase):
 
         # Test unknown resampling filter
         with hopper() as im:
-            self.assertRaises(ValueError, im.resize, (10, 10), "unknown")
+            with pytest.raises(ValueError):
+                im.resize((10, 10), "unknown")
 
     def test_default_filter(self):
         for mode in "L", "RGB", "I", "F":
             im = hopper(mode)
-            self.assertEqual(im.resize((20, 20), Image.BICUBIC), im.resize((20, 20)))
+            assert im.resize((20, 20), Image.BICUBIC) == im.resize((20, 20))
 
         for mode in "1", "P":
             im = hopper(mode)
-            self.assertEqual(im.resize((20, 20), Image.NEAREST), im.resize((20, 20)))
+            assert im.resize((20, 20), Image.NEAREST) == im.resize((20, 20))
+
+        for mode in "I;16", "I;16L", "I;16B", "BGR;15", "BGR;16":
+            im = hopper(mode)
+            assert im.resize((20, 20), Image.NEAREST) == im.resize((20, 20))
