@@ -1,9 +1,9 @@
 #!/bin/bash
 # install webp
 
-archive=libwebp-0.6.1
+archive=libwebp-1.2.1
 
-./download-and-extract.sh $archive https://raw.githubusercontent.com/python-pillow/pillow-depends/master/$archive.tar.gz
+./download-and-extract.sh $archive https://raw.githubusercontent.com/python-pillow/pillow-depends/main/$archive.tar.gz
 
 pushd $archive
 
